/**
 * This is a very simple applet that allows a user to experiment with a
 * single artificial neuron. The user can enter inputs and weights and
 * then have the activation value and neuron output displayed. As well, by
 * clicking on the activation function image, the user can cycle through
 * some standard functions. If the <shift> key is held down, a dialog is
 * displayed that allows the manipulation of some function parameters.
 *
 * @author  Fred Corbett
 * @version March 9, 1997 
 * modified 16 February 1998 Change so Legend text appears by default
 *  (Alix Herrmann).
 *  
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import anns.ActivationFunction;
import anns.ArtificialNeuron;

public class ArtificialNeuronApplet extends SimpleNeuronApplet implements KeyListener {


  /**********************/
  /* Instance variables */
  /**********************/

  /**
   * The McCulloch-Pitts neuron data model.
   */

  private ArtificialNeuron mpNeuron;

  /**
   *
   */

  private int width;

  /**
   *
   */

  private int height;

  /**
   *
   */

  private int legendXPos;


  /***************************/
  /* AWT Component Variables */
  /***************************/

  /**
   * When pressed, the textfields are read, the activation value and neuron
   * output are calculated, and the results are displayed.
   */

  private Button calculateButton;

  /**
   * When pressed, the textfields are cleared, and the activation function
   * is reset to UNIT_STEP.
   */

  private Button resetButton;


  /**
   * When pressed, the activation function is reset to UNIT_STEP and the weights
   * are set so neuron performs the AND function.
   */

  private Button andButton;

  /**
   * When pressed, the activation function is reset to UNIT_STEP and the weights
   * are set so neuron performs the OR function.
   */

  private Button orButton;

  /**
   * When pressed, the activation function is reset to UNIT_STEP and the weights
   * are set so neuron performs the NOT function. The second input/weight pair
   * is disabled.
   */

  private Button notButton;

  /**
   * When UNchecked, a legend is shown that labels each neuron element on the
   * screen.
   */

  private Checkbox showLegend;


  /******************/
  /* Public Methods */
  /******************/

    public ArtificialNeuronApplet() {}

  /**
   * Return applet information string.
   * @return String - Author, version information string.
   */

  public String getAppletInfo() {
    return "ArtificialNeuronApplet (March 9, 1997), by Fred Corbett - modified Feb 1998 (Alix Herrmann)";

  }


  /**
   * Initialize this applet.
   */

  public void init() {
    // Get the width and height parameters from the applet tag
    width = getSize().width;
    height = getSize().height;

    // Initialize panel/drawing locations based on applet size
    int stringWidth = getFontMetrics(getFont()).stringWidth("Activation Function") + 10;

    legendXPos = width - stringWidth;
    int panelHeight = new Button("Temp").getPreferredSize().height + 30;
    super.init(width- (stringWidth + 20),height - panelHeight,0,0);

    // Add the control panel
    Panel p = controlPanel();
    add(p);
    showLegend = new Checkbox("Hide legend");   
    p.add(showLegend);
    ItemListener itemListener = new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		repaint();
	    }
	};
    showLegend.addItemListener(itemListener);
    p.setBounds(0,height-panelHeight,width,panelHeight);

    // Instantiate a neuron, update the text fields and disable fields
    mpNeuron = new ArtificialNeuron(2,super.af);
    setNeuronFields(mpNeuron);
    mpNeuron.setThreshold(1f);
    neuronOutput.setEditable(false);
    weightedSum.setEditable(false);  

    threshold.addKeyListener(this);
    weight1.addKeyListener(this);
    weight2.addKeyListener(this);
    input1.addKeyListener(this);
    input2.addKeyListener(this);

    resize(width,height);
  }

    public void destroy() {
	this.afDialog.dispose();
    }
    public void stop() {
	this.afDialog.dispose();
    }

  /**
   * Paints the graphical elements of the applet.
   * @param Graphic g - the screen graphics context.
   */

  public void paint(Graphics g) {

    // Set the background color to erase or paint the legend text
      if (showLegend.getState()) {	
	  g.setColor(backgroundColor);
      }
      else {
	  g.setColor(Color.magenta);
      }

    // Draw arrows/lines to the neuron components
    drawArrow(g,legendXPos,noY+tfHH,noX+tfW,noY+tfHH);
    drawArrow(g,legendXPos,rY+cRad,rX+cDia,rY+cRad);
    g.drawLine(legendXPos,rY+cDia+arrowH/2,thX+tfW,rY+cDia+arrowH/2);
    drawArrow(g,thX+tfW,rY+cDia+arrowH/2,thX+tfHW,thY);
    drawArrow(g,legendXPos,cY+cRad,cX+cDia,cY+cRad);
    drawArrow(g,legendXPos,w1Y+tfHH,w1X+tfW,w1Y+tfHH);
    g.drawLine(legendXPos,w1Y+tfHH,w1X+tfHW,w1Y-(arrowH/2));
    drawArrow(g,w1X+tfHW,w1Y-(arrowH/2),w2X+tfW,w2Y+tfHH);
    drawArrow(g,legendXPos,i1Y+tfHH,i1X+tfW,i1Y+tfHH);
    g.drawLine(legendXPos,i1Y+tfHH,i1X+tfHW,i1Y-(arrowH/2));
    drawArrow(g,i1X+tfHW,i1Y-(arrowH/2),i2X+tfW,i2Y+tfHH);

    // Add the legend labels
    if (showLegend.getState())
      g.setColor(backgroundColor);
    else
      g.setColor(Color.black);
    g.drawString("Neuron Output",legendXPos + 5,noY+tfHH);
    g.drawString("Activation Function",legendXPos + 5,rY+cRad);
    g.drawString("Threshold",legendXPos + 5,rY+cDia+arrowH/2);
    g.drawString("Weighted Sum",legendXPos + 5,cY+cRad);
    g.drawString("Input Weights",legendXPos + 5,w1Y+tfHH);
    g.drawString("Input Values",legendXPos + 5,i1Y+tfHH);


    super.paint(g);
  }

  /**
   * Overidden to avoid blanking the screen.
   * @param Graphics g - the screen graphics context.
   */

  public void update(Graphics g) {
    paint(g);
  }

    /**
     * When the enter key is pressed we calculate the result.
     */
    public void keyPressed(KeyEvent e) {
	if (e.getKeyCode() == KeyEvent.VK_ENTER) {
	    clickedCalculateButton();
	}
    }
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}


  /*******************/
  /* Private Methods */
  /*******************/

  /**
   * Return a panel that displays the button components (using FlowLayout).
   * @return Panel - panel with button components.
   */

  private Panel controlPanel() {
    Panel p = new Panel();
    
    calculateButton = new Button("Calculate");
    ActionListener calculateListener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		clickedCalculateButton();
	    }
	};    
    calculateButton.addActionListener(calculateListener);
    p.add(calculateButton);    
    resetButton = new Button("Reset");
    ActionListener resetListener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		clickedResetButton();
	    } 
	};
    resetButton.addActionListener(resetListener);   
    p.add(resetButton);
    andButton = new Button("AND");
    ActionListener andListener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		clickedAndButton();
	    }
	};    
    andButton.addActionListener(andListener);
    p.add(andButton);  
    orButton = new Button("OR");
    ActionListener orListener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		clickedOrButton();
	    }
	};
    orButton.addActionListener(orListener);
    p.add(orButton);
    notButton = new Button("NOT");
    ActionListener notListener = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		clickedNotButton();
	    }
	};
    notButton.addActionListener(notListener);
    p.add(notButton);

    p.validate();
    return p;
  }

  /**
   * Respond to a Calculate-button press.
   */

  private void clickedCalculateButton() {
    getNeuronFields(mpNeuron);
    mpNeuron.calculate(getInputFields());
    setFieldToFloat(weightedSum, mpNeuron.getWeightedSum());
    setFieldToFloat(neuronOutput, mpNeuron.getOutput());
  }


  /**
   * Respond to a Reset-button press.
   */

  private void clickedResetButton() {
    weight2.setEnabled(true);
    input2.setEnabled(true);
    mpNeuron.clear();
    mpNeuron.setThreshold(0f);
    setNeuronFields(mpNeuron);
  }

  /**
   * Respond to an And-button press.
   */

  private void clickedAndButton() {
    changeActivationFunctionTo(ActivationFunction.UNIT_STEP);
    mpNeuron.setThreshold(2f);
    mpNeuron.setWeight(0,1f);
    mpNeuron.setWeight(1,1f);
    setNeuronFields(mpNeuron);
    weight2.setEnabled(true);
    input2.setEnabled(true);
  }

  /**
   * Respond to an Or-button press.
   */

  private void clickedOrButton() {
    changeActivationFunctionTo(ActivationFunction.UNIT_STEP);
    mpNeuron.setThreshold(1f);
    mpNeuron.setWeight(0, 1f);
    mpNeuron.setWeight(1, 1f);
    setNeuronFields(mpNeuron);
    weight2.setEnabled(true);
    input2.setEnabled(true);
  }

  /**
   * Respond to a Not-button press.
   */

  private void clickedNotButton() {
    changeActivationFunctionTo(ActivationFunction.UNIT_STEP);
    mpNeuron.setThreshold(0f);
    mpNeuron.setWeight(0, -1f);
    mpNeuron.setWeight(1, 0f);
    setNeuronFields(mpNeuron);
    weight2.setEnabled(false);
    input2.setEnabled(false);
  }


} // End of ArtificialNeuronApplet Class
